@@ -7,10 +7,10 @@ extends 'ELF::Writer';
 # ABSTRACT: ELF::Writer with defaults for Linux on x86_64
 
 
-has '+class_num'    => ( default => sub { 2 } );
-has '+data_num'     => ( default => sub { 1 } );
-has '+osabi_num'    => ( default => sub { 3 } );
-has '+machine_num'  => ( default => sub { 0x3E } );
+has '+class'    => ( default => sub { 2 } );
+has '+data'     => ( default => sub { 1 } );
+has '+osabi'    => ( default => sub { 3 } );
+has '+machine'  => ( default => sub { 0x3E } );
 
 
 sub _apply_segment_defaults {
@@ -33,7 +33,7 @@ ELF::Writer::Linux_x86_64 - ELF::Writer with defaults for Linux on x86_64
 
 =head1 VERSION
 
-version 0.000_001
+version 0.000_002
 
 =head1 DESCRPTION
 
